from ..exceptions import *
from .. import globals
import json


class GenerateQueryResultExportFileResult:
    def __init__(self, dict, client, debug):
        self._client = client
        self._debug = debug
        if dict is None:
            raise MissingDataInResultException
        else:
            self.buildFromDictionary(dict)

    def buildFromDictionary(self, dict):
        if dict is None:
            raise MissingDataInResultException

        self.queryId = dict["queryId"]
        self.timeSeriesDbId = dict["timeSeriesDbId"]
        self.fileFormat = dict["fileFormat"]

    @staticmethod
    def getQueryString(tabs=1, subobjectsRemaining=4):
        tabStr = globals.getTabStr(tabs)

        return f""" {{
{tabStr}queryId
{tabStr}timeSeriesDbId
{tabStr}fileFormat
{tabStr[0:-4]}}} """



from ..exceptions import *
from .. import globals


class ModelPropertyInput:

    def __init__(self, name, description, schemaType, defaultValue, writable, index):
        self.name = name
        self.description = description
        self.schemaType = schemaType
        self.defaultValue = defaultValue
        self.writable = writable
        self.index = index
        return

    def getMutationParameterString(self, tabs):
        tabStr = globals.getTabStr(tabs)
        result = f""" {{
{tabStr}name: "{self.name}",
{tabStr}{f'description: "{self.description}",' if self.description and self.description != None  else ""}
{tabStr}schemaType: {self.schemaType},
{tabStr}{f'defaultValue: "{self.defaultValue}",' if self.defaultValue and self.defaultValue != None else ""}
{tabStr}{f'writable: {self.writable},' if self.writable and self.writable != None else ""}
{tabStr}index: {self.index}
{tabStr[0:-4]}}} """
        return result

    @staticmethod
    def checkDictForCorrectFields(dict):
        if "name" not in dict:
            raise InvalidParameterException(
                "Model property entry is missing 'name'")
        if "schemaType" not in dict:
            raise InvalidParameterException(
                "Model property entry is missing 'schemaType'")
        elif dict["schemaType"] not in ['integer', 'double', 'string', 'boolean', 'dateTime', 'duration']:
            raise InvalidParameterException(
                f"""Property '{dict["name"]}' has invalid schemaType. Accepted values: ['integer', 'double', 'string', 'boolean', 'dateTime, 'duration']""")
        if "writable" in dict and type(dict["writable"]) is not bool:
            raise InvalidParameterException(
                "Model property 'writable' must be a boolean value")
        return True



from .mutation import *
from ..exceptions import *
from .. import globals
from ..data_types.digital_twin_model import DigitalTwinModel


class iot_createModel(Mutation):

    def __init__(self, client, debug, displayName, dtmiSegment, description, properties, telemetry):
        super().__init__(client, debug, "iot_createModel", DigitalTwinModel)
        self.displayName = displayName
        self.dtmiSegment = dtmiSegment
        self.description = description
        self.properties = properties
        self.telemetry = telemetry

    def _getMutationString(self):
        tabs = 1
        tabStr = globals.getTabStr(tabs)
        return f"""mutation {self.mutationName} {{ 
{tabStr}{self.mutationName}(
{tabStr}    displayName: "{self.displayName}", 
{tabStr}    dtmiSegment: "{self.dtmiSegment}", 
{tabStr}{'    description: "{self.description}",' if self.description != None else "" }
{tabStr}    properties: [{','.join([property.getMutationParameterString(tabs+1) for property in self.properties])}],
{tabStr}    telemetry: [{','.join([telem.getMutationParameterString(tabs+1) for telem in self.telemetry])}]) {self.resultType.getQueryString(tabs+1)} 
}}"""

    def performMutation(self):
        print(f"Creating model {self.displayName}...")
        self._result = self._client.performMutation(self._getMutationString())
        return self._processResult()

    def _processResult(self):
        super()._processResult()
        try:
            self.response = DigitalTwinModel(
                self._result['data'][self.mutationName], self._client, self._debug)
            print("Create succeeded")
            return self.response
        except Exception as e:
            raise e



from .mutation import *
from ..exceptions import *
from .. import globals
from ..data_types.iot_endpoint import IotEndpoint


class iot_createEndpoint(Mutation):

    def __init__(self, client, debug, iotHubId, modelId, name, description):
        super().__init__(client, debug, "iot_createEndpoint", IotEndpoint)
        self.iotHubId = iotHubId
        self.modelId = modelId
        self.name = name
        self.description = description

    def _getMutationString(self):
        tabs = 1
        tabStr = globals.getTabStr(tabs)
        return f"""mutation {self.mutationName} {{ 
{tabStr}{self.mutationName}(
{tabStr}    iotHubId: "{self.iotHubId}", 
{tabStr}    modelId: "{self.modelId}", 
{tabStr}    name: "{self.name}", 
{tabStr}{'    description: "{self.description}"' if self.description != None else ''}) {self.resultType.getQueryString(tabs+1)} 
}}"""

    def performMutation(self):
        print(f"Creating endpoint {self.name}...")
        self._result = self._client.performMutation(self._getMutationString())
        return self._processResult()

    def _processResult(self):
        super()._processResult()
        try:
            self.response = IotEndpoint(
                self._result['data'][self.mutationName], self._client, self._debug)
            print("Create endpoint succeeded")
            return self.response
        except Exception as e:
            raise e



from ..exceptions import *
from .. import globals


class ModelTelemetryInput:
    def __init__(self, name, description, schemaType, index):
        self.name = name
        self.description = description
        self.schemaType = schemaType
        self.index = index
        return

    def getMutationParameterString(self, tabs):
        tabStr = globals.getTabStr(tabs)
        result = f""" {{
{tabStr}name: "{self.name}",
{tabStr}{f'description: "{self.description}",' if self.description != None else ""}
{tabStr}schemaType: {self.schemaType},
{tabStr}index: {self.index}
{tabStr[0:-4]}}} """
        return result

    @staticmethod
    def checkDictForCorrectFields(dict):
        if "name" not in dict:
            raise InvalidParameterException(
                "Model telemetry entry is missing 'name'")
        if "schemaType" not in dict:
            raise InvalidParameterException(
                "Model telemetry entry is missing 'schemaType'")
        return True
